/*
 * @Author: wjc
 * @Date: 2024-06-05 10:21:23
 * @LastEditors: wjc
 * @LastEditTime: 2024-07-01 15:24:39
 * @Description:
 */
import type { App } from 'vue'
import qs from 'qs'
import type { ILogin } from '@/models/userTypes'
import { useUserStore } from '@/stores/modules/userStore'
import { checkStatus } from './checkStatus'

export type IRequestOptions = UniApp.RequestOptions & {
  query?: Record<string, any>
}

const interceptor = {
  //  请求前拦截
  invoke(options: IRequestOptions) {
    const userStore = useUserStore()
    // api 处理
    options.url = `${import.meta.env.VITE_APP_BASE_API}${options.url}`
    // 查询参数处理
    if (options.query) {
      const query = qs.stringify(Object.assign({}, options.query), {
        addQueryPrefix: true,
      })
      options.url += query
    }
    // 10 秒请求超时
    options.timeout = 100000
    // 请求头标识符
    const sysInfo = uni.getSystemInfoSync()
    const storageLoginInfo: ILogin = userStore.storageLoginInfo
    options.header = {
      ...options.header,
      version: sysInfo.appVersion, // 版本号
      platform: sysInfo.osName ?? 'app', // 所用系统
    }
    // entCode
    if (storageLoginInfo && storageLoginInfo.entCode) {
      options.header.entcode = storageLoginInfo.entCode
    }
    // token
    if (userStore.token) {
      options.header.Authorization = `Bearer ${userStore.token}`
    }
  },
}

export function setupInterceptor(app: App) {
  app.use({
    install() {
      uni.addInterceptor('request', interceptor)
    },
  })
}

export const request = <T>(options: IRequestOptions) => {
  return new Promise<T>((resolve, reject) => {
    uni.request({
      responseType: 'json',
      ...options,
      success(res) {
        if (res.statusCode === 200 || res.statusCode === 201) {
          resolve(res.data as T)
        } else {
          checkStatus(res)
          reject(res)
        }
      },
    })
  })
}

request.get = <T>(url: string, options: Omit<IRequestOptions, 'url'> = {}) => {
  return request<T>({
    method: 'GET',
    url,
    ...options,
  })
}

request.post = <T>(url: string, options: Omit<IRequestOptions, 'url'> = {}) => {
  return request<T>({
    method: 'POST',
    url,
    ...options,
  })
}

request.put = <T>(url: string, options: Omit<IRequestOptions, 'url'> = {}) => {
  return request<T>({
    method: 'PUT',
    url,
    ...options,
  })
}

request.del = <T>(url: string, options: Omit<IRequestOptions, 'url'> = {}) => {
  return request<T>({
    method: 'DELETE',
    url,
    ...options,
  })
}

export const uploadFile = <T>({ tempFilePath, formData, loading, error, data }) => {
  uni.uploadFile({
    url: import.meta.env.VITE_APP_UPLOAD_API,
    filePath: tempFilePath,
    name: 'file',
    formData,
    success: (uploadFileRes) => {
      data.value = uploadFileRes.data as T
    },
    fail: (err) => {
      error.value = true
    },
    complete: () => {
      loading.value = false
    },
  })
}

request.uploadFile = <T>(formData: Record<string, any> = {}, count = 1) => {
  const loading = ref(false)
  const error = ref(false)
  const data = ref<T>()

  // #ifdef MP-WEIXIN
  // 微信小程序从基础库 2.21.0 开始， wx.chooseImage 停止维护，请使用 uni.chooseMedia 代替。
  uni.chooseMedia({
    count: 1,
    mediaType: ['image'],
    success: (res) => {
      loading.value = true
      const tempFilePath = res.tempFiles[0].tempFilePath
      uploadFile<T>({ tempFilePath, formData, data, error, loading })
    },
    fail: (err) => {
      console.error('uni.chooseMedia err->', err)
      error.value = true
    },
  })
  // #endif
  // #ifndef MP-WEIXIN
  uni.chooseImage({
    count,
    success: (res) => {
      loading.value = true
      const tempFilePath = res.tempFilePaths[0]
      uploadFile<T>({ tempFilePath, formData, data, error, loading })
    },
    fail: (err) => {
      console.error('uni.chooseImage err->', err)
      error.value = true
    },
  })
  // #endif

  return { loading, error, data }
}
